/*
* j1939_datatype.h - contains defines for data types
*
* Copyright (c) 2014-2020 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_datatype.h 30689 2020-02-14 16:09:23Z boe $
*-------------------------------------------------------------------
*
*
*
*/

/**
* \brief data types
*
* \file j1939_datatype.h - contains defines for data types
*
*/

#ifndef J1939_DATATYPE_H
#define J1939_DATATYPE_H 1


/* define of masks */
#define J1939_PGN_PRIORITY_MASK ~(0x3fffffful)

#define J1939_PGN_PRIORITY_0 0x00000000ul /**< bitmask for priority 0 */
#define J1939_PGN_PRIORITY_1 0x04000000ul /**< bitmask for priority 1 */
#define J1939_PGN_PRIORITY_2 0x08000000ul /**< bitmask for priority 2 */
#define J1939_PGN_PRIORITY_3 0x0c000000ul /**< bitmask for priority 3 */
#define J1939_PGN_PRIORITY_4 0x10000000ul /**< bitmask for priority 4 */
#define J1939_PGN_PRIORITY_5 0x14000000ul /**< bitmask for priority 5 */
#define J1939_PGN_PRIORITY_6 0x18000000ul /**< bitmask for priority 6 */
#define J1939_PGN_PRIORITY_7 0x1c000000ul /**< bitmask for priority 7 */


#ifndef ALIGNMENT
# define ALIGNMENT  2
#endif /* ALIGNMENT */

#ifndef U_DSP_T
# if ALIGNMENT == 2
typedef unsigned char U_DSP_T;
# endif /* ALIGNMENT == 2 */
# if ALIGNMENT == 4
typedef unsigned int U_DSP_T;
# endif /* ALIGNMENT == 4 */
#endif


/**
 * J1939 data types
 */
typedef enum {
	J1939_DTYPE_U1, J1939_DTYPE_U2, J1939_DTYPE_U3, J1939_DTYPE_U4,
	J1939_DTYPE_U5, J1939_DTYPE_U6, J1939_DTYPE_U7, J1939_DTYPE_U8,
	J1939_DTYPE_U12, J1939_DTYPE_U16, J1939_DTYPE_U24, J1939_DTYPE_U32,
	J1939_DTYPE_U40, J1939_DTYPE_U48, J1939_DTYPE_U64,
	J1939_DTYPE_NON_NUMERIC = 256,
	J1939_DTYPE_STRING_ASTERIKS,
	J1939_DTYPE_STRING_NULL
} J1939_DATA_TYPE_T;

/**
 * Device Name
 */
typedef struct {
	UNSIGNED32	identifyNumber;		/**< identify number */
	UNSIGNED16	manufacturerCode;	/**< manufacturer code */
	UNSIGNED8	ecuInstance;		/**< ecu instance */
	UNSIGNED8	functionInstance;	/**< function instance */
	UNSIGNED8	function;			/**< function */
	UNSIGNED8	vehicleSystem;		/**< vehicle system */
	UNSIGNED8	vehicleInstance;	/**< vehicle instance */
	UNSIGNED8	industrieGroup;		/**< industrie group */
	UNSIGNED8	adressCapable;		/**< address capable */
} J1939_DEVICE_NAME;


/**
 * Direction
 */
typedef enum {
	DIR_RECEIVE,
	DIR_TRANSMIT
} DIRECTION_T;


/**
 * mapping entries
 */
typedef struct {
	void			*pVar;			/**< Pointer auf Variable */
	UNSIGNED16		nrOfBits;		/**< number of bits for this variable */
	UNSIGNED32		spn;			/**< spn number */
	J1939_DATA_TYPE_T	dType;		/**< data type */
#ifdef J1939_DYNAMIC_PGN
	UNSIGNED8		dynVar;			/**< dynamic variable */
#endif /*J1939_DYNAMIC_PGN*/
} J1939_MAPPING_T;


/**
 * PGN definition 
 */
typedef struct {
	UNSIGNED32	pgn;				/**< PGN number */
#ifdef J1939_DYNAMIC_PGN
	J1939_MAPPING_T *pMap;			/**< pointer to mapping */
#else /*J1939_DYNAMIC_PGN*/
	const J1939_MAPPING_T *pMap;	/**< pointer to mapping */
#endif /*J1939_DYNAMIC_PGN*/
	UNSIGNED16	cycleTime;			/**< cycle time in msec (0 for acyclic) */
	UNSIGNED16	inhibitTime;		/**< inhibit in msec */
	UNSIGNED8	priority;			/**< PGN priority */
	UNSIGNED8	nrOfMappings;		/**< nr of mappings */
} J1939_PGN_CONFIG_T;


/**
 * Receive State
 * (used for receive indication)
 */
typedef enum {
	J1939_RECEIVE_STATE_ACTIVE,		/**< receive PGN is active */
	J1939_RECEIVE_STATE_FAILURE		/**< receive PGN has been failed */
} J1939_RECEIVE_STATE_T;


/**
 * stack initialisation
 */
typedef struct {
	const J1939_PGN_CONFIG_T	*trPgnConfig;	/**< pointer to transmit PGNs */
	const UNSIGNED16			trPgnCnt;		/**< number of transmit PGNs */
	const J1939_PGN_CONFIG_T	*recPgnConfig;	/**< pointer to receive PGNs */
	const UNSIGNED16			recPgnCnt;		/**< number of receive PGNs */
	const J1939_PGN_CONFIG_T	*devNameConfig;	/**< device name */
	UNSIGNED8					nodeAddress;	/**< first claim address */
} J1939_CONFIG_T;

#endif /* J1939_DATATYPE_H */

